#!/bin/bash
#set -n		# read but do not execute commands (check on syntax errors)
#
# File: /usr/local/sbin/gpgswitchd.sh
#
# This is the GPGswitch control program.
#
#    Copyright (C) 2012  Juergen Kaesmann (JK)
#
##############################################################################
# GPL NOTICE
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

#. /usr/local/etc/main.conf				# config main
#. $LLIBDIR/libfuncstd-sh				# functions standard
#. /usr/local/share/ksm-gpgswitch/libfuncstd-sh		# functions standard
#. /usr/local/etc/gpgswitch.conf			# config gpgswitch
#. ~/.samplerc						# config user

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

#------------------STANDARD CONSTANTS AND VARIABLES---------------------------
readonly package=GPGswitch				# package name
readonly vinfopath='/usr/local/share/ksm-gpgswitch'	# path to package dir
readonly version=$(/bin/cat $vinfopath/VERSION)		# package version
readonly release=$(/bin/cat $vinfopath/RELEASE)		# package release
readonly scriptname=$(basename $0)			# name of script
TEMPDIR='/tmp'
readonly tempfile=$TEMPDIR/$scriptname.$UID.tmp		# temporary file name
#tempfile=$(mktemp $TEMPDIR/$scriptname.$UID.XXXXXX)	# temporary file name
readonly debuglog="/bin/logger -p user.debug -t $scriptname -- DEBUG:"	# log dbg msg
readonly errlog="/bin/logger -p user.err -t $scriptname -- ERROR:"	# log err msg
readonly infolog="/bin/logger -p user.info -t $scriptname -- INFO:"	# log inf msg
cmd=''							# sub command
debug=no						# debug=yes/no
verbose=no						# verbose=yes/''/no

#------------------USER FILL IN AFTER INSTALLATION----------------------------
#echo "$scriptname line 60: Setup following 2 vars. Then comment out this line." >&2; exit 1
#readonly var1='Dummy1'
#var2='Dummy2'

#-----------------------USER-CONSTANTS----------------------------------------
# STRINGS
#readonly const=string

# NUMBERS
#readonly const=value

#-----------------------USER-VARIABLES----------------------------------------
# STRINGS
mfile=/tmp/$scriptname.$UID.m.tmp	# mail
hostname=`/bin/hostname --fqdn`		# FQDN

#args=''				# all args from cmdline
#arg1=''				# 1. arg from cmdline
#opt_i=''				# option -i (arg)
#opt_o=''				# option -o (yes)
#sample-opt=''				# option --sample-opt (yes/no/arg)
#infile=''				# option --infile (arg)
#outfile=''				# option --outfile (arg)

# NUMBERS
err=0					# error code

# ARRAYS
# Indexed arrays are always zero based string arrays.
# If index starts with 1 and all members are assigned then
# count of members of array = last index of array
#declare -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#declare -a ErrMsg			# type is array of strings

##############################################################################
# LANGUAGE SUPPORT

Lang=${LANG%.*}						# rm character set
Lang=${Lang%_*}						# get 2 char lang code
if [ -e $vinfopath/lang/$scriptname.$Lang ]; then	# lang pkg actual
	. $vinfopath/lang/$scriptname.$Lang
elif [ -e $vinfopath/lang/$scriptname.en ]; then	# lang pkg en
	. $vinfopath/lang/$scriptname.en
else							# no lang support
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[5]} No language support." >&2
	else
		$errlog ${ErrMsg[5]} No language support.
		# Send a mail with mailx, Format:
		# echo "TEXT" | mailx -s "SUBJECT" -r "SENDER" RECIPIENT
		echo "$criptname: No language support - Abort!" | \
			mailx -s "$criptname - Abort" -r $criptname root
	fi
	exit 1
fi

##############################################################################
# FUNCTIONS
##############################################################################

#=============================================================================
function helptext ()
{
# Output helptext
# Uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG helptext: $# $@" >&2
	$debuglog helptext: $# $@
fi

echo "$LangMSG_helptext"

return
}

#=============================================================================
function mainproc ()
{
# This function controls gpgswitchdcat while a connection has established.
# par1 = 
# Uses cat, chmod, cp, echo, gpgswitch, grep, logger, su
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG main proc: $# $@" >&2
	$debuglog main proc: $# $@
fi

local sender='' rcpt='' s='' s1='' s2='' ss=''
local l=0 ret=0
#local -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#s=$(date "+%d.%m.%Y  %H:%M:%S")	# DD.MM.YYYY  HH:MM:SS

echo "220 $hostname GPGswitchD"				# send greeting

while read s; do
	l=${#s}						# l = len input str
	if [ "$l" -gt 0 ]; then
		let --l
	fi
	s="${s:0:$l}"					# rm LF

	s1="${s%% *}"					# get 1. word
	s1=${s1^^}
	case "$s1" in
	  DATA)					# input full mail, end with .
		if [ -n "$sender" ] && [ -n "$rcpt" ]; then
			echo "354 End data with <CR><LF>.<CR><LF>"

			cp /dev/null $mfile
			chmod 644 $mfile
			while read s; do
				l=${#s}			# l = len input str
				let --l
				s="${s:0:$l}"		# rm LF
				if [ "$s" == "." ]; then	# EOD
					break
				fi
				echo "$s" >> $mfile
			done
			echo "250 Data Ok: piped to gpgswitch"

			# give mail to gpgswitch
			/bin/su $sender -c "/bin/cat $mfile | /usr/local/bin/gpgswitch -o -oi -- $rcpt"
			sender=''
			rcpt=''
		else
			echo "503 Bad sequence of commands"
			$errlog 503 Bad sequence of commands
		fi
		;;
	  EHLO)						# EHLO host.domain.tld
		echo "250 $hostname"
		;;
	  HELO)						# HELO host.domain.tld
		echo "250 $hostname"
		;;
	  HELP)						# HELP
		echo "214-DATA, EHLO, HELO, HELP, MAIL FROM:"
		echo "214 NOOP, QUIT, RCPT TO:, RSET"
		;;
	  MAIL)					# MAIL FROM: <name@domain.de>
		# get 2. word 'FROM:'
		ss="${s#* }"				# rm 1. word
		s2="${ss%%:*}"				# rm all after 2. word
		s2="${s2}:"				# app removed :
		s2=${s2^^}
		if [ "$s2" == "FROM:" ]; then
			ss="${s#*<}"
			ss="${ss%>*}"
			sender="${ss%@*}"

			# check wether sender is local
			/usr/bin/grep "^${sender}:" /etc/passwd > /dev/null 2>&1
			ret=$?
			if [ "$ret" -eq 0 ]; then
				echo "250 MAIL FROM: Ok"
			else
				echo "551 User not local"
				$errlog 551 User not local
				sender=''
			fi
		else
			echo "500 Syntax error, command unrecognized: ${s1} ${2}"
			$errlog 500 Syntax error, command unrecognized: ${s1} ${2}
		fi
		;;
	  NOOP)						# NOOP
		echo "250 Ok"
		;;
	  QUIT)						# QUIT
		echo "221 Bye"
		break
		;;
	  RCPT)					# RCPT TO: <name@domain.de>
		# get 2. word 'TO:'
		ss="${s#* }"				# rm 1. word
		s2="${ss%%:*}"				# rm all after 2. word
		s2="${s2}:"				# app removed :
		s2=${s2^^}
		if [ "$s2" == "TO:" ]; then
			ss="${s#*<}"
			rcpt="${rcpt} ${ss%>*}"
			echo "250 RCPT TO: Ok"
		else
			echo "500 Syntax error, command unrecognized: ${s1} ${s2}"
			$errlog 500 Syntax error, command unrecognized: ${s1} ${s2}
		fi
		;;
	  RSET)						# RSET
		sender=''
		rcpt=''
		echo "250 RSET Ok"
		;;
	  *)						# ERROR
		echo "500 Syntax error, command unrecognized: ${s1}"
		$errlog 500 Syntax error, command unrecognized: ${s1}
		;;
	esac
done
rm -f $mfile
ret=0

return $ret
}

##############################################################################
# GET OPTIONS
##############################################################################
# Give 2 strings and all cmdline_args to func Get Opt
# (POSIX_options Gnu_long_options cmdline_args):
# Get Opt 'hVdqvi:m:' 'help version debug quiet verbose infile= myval=' $@
#
# A ':' after a POSIX option means this option needs an arg.
# A '=' after a Gnu long option means this option needs an arg.
#
# All cmd/options have to be initialised to ''!
#
# POSIX options are classified in 2 categories:
#   a) cmds (mutually exclusive). All cmds are given to var 'cmd'.
#   b) options (as many as you like). Each option is given to it's own var.
# As a general rule: If cmds/options are given multiple then last given wins.
#
# Options can take 1 argument (no whitespace in arg). If you want to give more
# than 1 arg or want to use multiple strings as 1 arg then concatenate them
# together with a delimiter like ',' or ';' or '.'
# Use a delimiter which will not be part of the arg:
# sampleprog -m val1,val2
# sampleprog --myval=val1,val2
# It's at the programmers responsibility to do meaningfull things with the
# content of the options.
#
# If err != 0 then all args are processed, else continue to look for options.
# When Get Opt finishes then all options are removed from cmdline.
#
if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Parse Commandline: $# $@" >&2
	$debuglog Parse Commandline: $# $@
fi
#
err=0
while [ "$err" -eq 0 ]; do
	GetOpt 'hVd' 'help version debug' $@
	err=$?
	if [ $err -eq 0 ]; then
		case "$_OPTNAME_" in
		    h|help)	cmd="-h";;
		    V|version)	cmd="-V";;
		    d|debug)	debug=yes;;
#		    q|quiet)	verbose=no;;
#		    v|verbose)	verbose=yes;;
#		    o|output)	opt_o=yes;;
#		    i|infile)	opt_i="$_OPTVAL_";;
		    --)		break;;
		    :)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[8]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[8]} -${_OPTVAL_}
			fi; exit 8;;
		    ?)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[4]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[4]} -${_OPTVAL_}
			fi; exit 4;;
		    *)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}
			fi; exit 4;;
		esac
	fi
done
err=0
if [ -z "$cmd" ]; then					# error no cmd given
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[7]}" >&2
	else
		$errlog ${ErrMsg[7]}
	fi
	exit 7
fi
shift ${_OPTIDX_}				# rm all opts from cmdline

##############################################################################
# CHECK LINK NAMES

##############################################################################
# CHECK OPTIONS

##############################################################################
# CHECK VERSION
#
# check version of config files. Conf file = global, Rc file = user specific
#if [ "$version" != "$Sample_Conf" ] || [ "$version" != "$Sample_Rc" ]; then
if [ "$version" != "$GPGswitch_Conf" ]; then
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[12]}" >&2
	else
		$errlog ${ErrMsg[12]}
	fi
	exit 12
fi

##############################################################################
# CHECK ARGUMENTS
#
case "$cmd" in
#    '')
#	if [ "$#" -eq 1 ]; then				# parameter count ok
#		arg1="$1"
#		shift
#		#args="$@"
#		#shift $#
#	else						# error in param count
#		if [ "$verbose" != no ]; then
#			echo "${ErrMsg[2]}" >&2
#		else
#			$errlog ${ErrMsg[2]}
#		fi
#		exit 2
#	fi
#	;;
    *)
	if [ "$#" -ne 0 ]; then				# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 2
	fi
	;;
esac

##############################################################################
# MAINPROGRAM
##############################################################################
#
if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG Main: $# $@" >&2
	$debuglog Main: $# $@
fi

# Check what to do (initial cmds, options and args are removed from cmdline)
case "$cmd" in
    '')							# no cmd given
	mainproc
	err=$?
	;;
    -h)							# cmd -h
	helptext
	err=0
	;;
    -V)							# cmd -V
	echo "$scriptname (${package}) ${version}-${release}"
	err=0
	;;
    *)							# other cmds = error
	$errlog Command not in command list
	err=6
	;;
esac

exit $err
