#!/bin/bash
#set -n		# read but do not execute commands (check on syntax errors)
#
# File: /usr/local/sbin/keymand.sh
#
# Manage keys for GPGswitch.
#
#    Copyright (C) 2015  Juergen Kaesmann (JK)
#
##############################################################################
# GPL NOTICE
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

#. /usr/local/etc/main.conf				# config main
#. $LLIBDIR/libfuncstd-sh				# functions standard
. /usr/local/share/ksm-gpgswitch/libfuncstd-sh		# functions standard
. /usr/local/etc/gpgswitch.conf				# config gpgswitch
#. ~/.samplerc						# config user

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

#------------------STANDARD CONSTANTS AND VARIABLES---------------------------
readonly package=GPGswitch				# package name
readonly vinfopath='/usr/local/share/ksm-gpgswitch'	# path to package dir
readonly version=$(/bin/cat $vinfopath/VERSION)		# package version
readonly release=$(/bin/cat $vinfopath/RELEASE)		# package release
readonly scriptname=$(basename $0)			# name of script
TEMPDIR='/tmp'
readonly tempfile=$TEMPDIR/$scriptname.$UID.tmp		# temporary file name
#tempfile=$(mktemp $TEMPDIR/$scriptname.$UID.XXXXXX)	# temporary file name
readonly debuglog="/bin/logger -p user.debug -t $scriptname -- DEBUG:"	# log dbg msg
readonly errlog="/bin/logger -p user.err -t $scriptname -- ERROR:"	# log err msg
readonly infolog="/bin/logger -p user.info -t $scriptname -- INFO:"	# log inf msg
cmd=''							# sub command
debug=no						# debug=yes/no
verbose=''						# verbose=yes/''/no

#------------------USER FILL IN AFTER INSTALLATION----------------------------
#echo "$scriptname line 60: Setup following 2 vars. Then comment out this line." >&2; exit 1
#readonly var1='Dummy1'
#var2='Dummy2'

#-----------------------USER-CONSTANTS----------------------------------------
# STRINGS
#readonly const=string
readonly pidfile=/var/run/${scriptname}.pid		# the pidfile
readonly pipe="$KeymanPipe"				# the pipe for keymand
readonly keyfile=/usr/local/etc/gpgkeys.asc		# file with all keys

# NUMBERS
#readonly const=value

#-----------------------USER-VARIABLES----------------------------------------
# STRINGS
#args=''				# all args from cmdline
arg1=''					# 1. arg from cmdline
#opt_i=''				# option -i (arg)
#opt_o=''				# option -o (yes)
#sample-opt=''				# option --sample-opt (yes/no/arg)
#infile=''				# option --infile (arg)
#outfile=''				# option --outfile (arg)

# NUMBERS
err=0					# error code

# ARRAYS
# Indexed arrays are always zero based string arrays.
# If index starts with 1 and all members are assigned then
# count of members of array = last index of array
#declare -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#declare -a ErrMsg			# type is array of strings

##############################################################################
# LANGUAGE SUPPORT

Lang=${LANG%.*}						# rm character set
Lang=${Lang%_*}						# get 2 char lang code
if [ -e $vinfopath/lang/$scriptname.$Lang ]; then	# lang pkg actual
	. $vinfopath/lang/$scriptname.$Lang
elif [ -e $vinfopath/lang/$scriptname.en ]; then	# lang pkg en
	. $vinfopath/lang/$scriptname.en
else							# no lang support
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[5]} No language support." >&2
	else
		$errlog ${ErrMsg[5]} No language support.
		# Send a mail with mailx, Format:
		# echo "TEXT" | mailx -s "SUBJECT" -r "SENDER" RECIPIENT
		echo "$criptname: No language support - Abort!" | \
			mailx -s "$criptname - Abort" -r $criptname root
	fi
	exit 1
fi

##############################################################################
# SIGNAL HANDLING

# 1)SIGHUP	 2) SIGINT	 3) SIGQUIT	 4) SIGILL	 5) SIGTRAP
# 6)SIGABRT	 7) SIGBUS	 8) SIGFPE	 9) SIGKILL	10) SIGUSR1
#11)SIGSEGV	12) SIGUSR2	13) SIGPIPE	14) SIGALRM	15) SIGTERM
#16)SIGSTKFLT	17) SIGCHLD	18) SIGCONT	19) SIGSTOP	20) SIGTSTP
#21)SIGTTIN	22) SIGTTOU	23) SIGURG	24) SIGXCPU	25) SIGXFSZ
#26)SIGVTALRM	27) SIGPROF	28) SIGWINCH	29) SIGIO	30) SIGPWR
#31)SIGSYS	34) SIGRTMIN	35) SIGRTMIN+1	36) SIGRTMIN+2	37) SIGRTMIN+3
#38)SIGRTMIN+4	39) SIGRTMIN+5	40) SIGRTMIN+6	41) SIGRTMIN+7	42) SIGRTMIN+8
#43)SIGRTMIN+9	44) SIGRTMIN+10	45) SIGRTMIN+11	46) SIGRTMIN+12	47) SIGRTMIN+13
#48)SIGRTMIN+14	49) SIGRTMIN+15	50) SIGRTMAX-14	51) SIGRTMAX-13	52) SIGRTMAX-12
#53)SIGRTMAX-11	54) SIGRTMAX-10	55) SIGRTMAX-9	56) SIGRTMAX-8	57) SIGRTMAX-7
#58)SIGRTMAX-6	59) SIGRTMAX-5	60) SIGRTMAX-4	61) SIGRTMAX-3	62) SIGRTMAX-2
#63)SIGRTMAX-1	64) SIGRTMAX	 0) EXIT	

# signal KILL will be immediately executed from the shell, no trap

#=============================================================================
#function load_conf ()
#{
#source /usr/local/etc/gpgswitch.conf			# config gpgswitch
#return
#}

#=============================================================================
function exit_prog ()
{
# do exit stuff here like: flush buffers, close files, eg.
echo "QUIT" > $pipe
#exit
}

#=============================================================================
#trap load_conf 1					# reload config
trap exit_prog 15					# do smart exit

##############################################################################
# FUNCTIONS
##############################################################################

#=============================================================================
function helptext ()
{
# Output helptext
# Uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	$debuglog helptext: $# $@
fi

echo "$LangMSG_helptext"

return
}

#=============================================================================
function handle_transaction ()
{
# Make the auth process and handle the transaction.
# par1 = keys
# Uses cut, echo, gpg, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	$debuglog handle_transaction: $#
fi

local cert='' certmail='' email='' found=no keys="$1" s='' sadr='' skey=''
local ret=0

echo AUTH > $pipe
cert="$(/bin/cat $pipe)"
echo KEYID > $pipe
email="$(/bin/cat $pipe)"

echo "$cert" > $tempfile
s=$(/usr/bin/gpg --keyring /root/.gnupg/pubring.gpg --verify $tempfile 2>&1)
ret=$?
rm -f $tempfile
if [ "$ret" -eq 0 ]; then
	certmail=$(echo $s | /usr/bin/cut -d '<' -f 2 \
		| /usr/bin/cut -d '>' -f 1)		# get email from cert
	if [ "$email" == "$certmail" ]; then	# request mail is same as cert
		found=no
		for s in $keys; do
			sadr=${s%;*}
			skey=${s#*;}
			if [ "$sadr" == "$email" ]; then
				found=yes
				break
			fi
		done
		if [ "$found" == yes ]; then		# pass for keyid found
			echo "$skey" > $pipe
		else
			$errlog keyid not found in configured keys
			echo ERROR > $pipe		# keyid not found
		fi
	else
		$errlog keyid and cert do not match - keyid=$email cert-id=$certmail
		echo ERROR > $pipe
	fi
else
	$errlog verification of cert failed - keyid from client = $email
	echo ERROR > $pipe
fi

return $ret
}

#=============================================================================
function mainproc ()
{
# Key manager handling
# par1 = 
# Uses cat, chmod, logger, mkfifo, rm, togglegpgswitch
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	$debuglog main proc: $# $@
fi

local keys='' pstr='' quit=no
local i=0 ret=0
#local -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array
#s=$(date "+%d.%m.%Y  %H:%M:%S")	# DD.MM.YYYY  HH:MM:SS

# store pid
echo "$$" > $pidfile

# make pipe
/usr/bin/mkfifo $pipe
/bin/chmod 666 $pipe

keys=$(cat $arg1)
/bin/rm -f $arg1
arg1=''

/usr/local/bin/togglegpgswitch -e -q

# read str from pipe
quit=no
while [ "$quit" == no ]; do
	pstr="$(/bin/cat $pipe)"
	case "$pstr" in
	    QUIT)
		quit=yes
		;;
	    KEY)
		handle_transaction "$keys"
		;;
	    *)
		$errlog unkwon pstr=$pstr
		;;
	esac
done

keys=''
/usr/local/bin/togglegpgswitch -d -q
/bin/rm -f $pidfile
/bin/rm -f $pipe

return $ret
}

##############################################################################
# GET OPTIONS
##############################################################################
# Give 2 strings and all cmdline_args to func Get Opt
# (POSIX_options Gnu_long_options cmdline_args):
# Get Opt 'hVdqvi:m:' 'help version debug quiet verbose infile= myval=' $@
#
# A ':' after a POSIX option means this option needs an arg.
# A '=' after a Gnu long option means this option needs an arg.
#
# All cmd/options have to be initialised to ''!
#
# POSIX options are classified in 2 categories:
#   a) cmds (mutually exclusive). All cmds are given to var 'cmd'.
#   b) options (as many as you like). Each option is given to it's own var.
# As a general rule: If cmds/options are given multiple then last given wins.
#
# Options can take 1 argument (no whitespace in arg). If you want to give more
# than 1 arg or want to use multiple strings as 1 arg then concatenate them
# together with a delimiter like ',' or ';' or '.'
# Use a delimiter which will not be part of the arg:
# sampleprog -m val1,val2
# sampleprog --myval=val1,val2
# It's at the programmers responsibility to do meaningfull things with the
# content of the options.
#
# If err != 0 then all args are processed, else continue to look for options.
# When Get Opt finishes then all options are removed from cmdline.
#
if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Parse Commandline: $# $@" >&2
	$debuglog Parse Commandline: $# $@
fi
#
err=0
while [ "$err" -eq 0 ]; do
	GetOpt 'hVdqv' 'help version debug quiet verbose' $@
	err=$?
	if [ $err -eq 0 ]; then
		case "$_OPTNAME_" in
		    h|help)	cmd="-h";;
		    V|version)	cmd="-V";;
#		    a)		cmd="-${_OPTNAME_}";;
		    d|debug)	debug=yes;;
		    q|quiet)	verbose=no;;
		    v|verbose)	verbose=yes;;
#		    o|output)	opt_o=yes;;
#		    i|infile)	opt_i="$_OPTVAL_";;
		    --)		break;;
		    :)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[8]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[8]} -${_OPTVAL_}
			fi; exit 8;;
		    ?)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[4]} -${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[4]} -${_OPTVAL_}
			fi; exit 4;;
		    *)	if [ "$verbose" != no ]; then
				echo "${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}" >&2
			else
				$errlog ${ErrMsg[99]} _OPTNAME_=${_OPTNAME_} _OPTVAL_=${_OPTVAL_}
			fi; exit 4;;
		esac
	fi
done
err=0
#if [ -z "$cmd" ]; then					# error no cmd given
#	if [ "$verbose" != no ]; then
#		echo "${ErrMsg[7]}" >&2
#	else
#		$errlog ${ErrMsg[7]}
#	fi
#	exit 7
#fi
shift ${_OPTIDX_}				# rm all opts from cmdline

##############################################################################
# CHECK LINK NAMES

##############################################################################
# CHECK OPTIONS

##############################################################################
# CHECK VERSION

##############################################################################
# CHECK ARGUMENTS
#
case "$cmd" in
    '')
	if [ "$#" -eq 1 ]; then				# parameter count ok
		arg1="$1"
		shift
		#args="$@"
		#shift $#
	else						# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 2
	fi
	;;
    *)
	if [ "$#" -ne 0 ]; then				# error in param count
		if [ "$verbose" != no ]; then
			echo "${ErrMsg[2]}" >&2
		else
			$errlog ${ErrMsg[2]}
		fi
		exit 2
	fi
	;;
esac

##############################################################################
# MAINPROGRAM
##############################################################################
#
if [ "$debug" == "yes" ]; then				# DEBUG
	$debuglog Main: $# $@
fi

# Check what to do (subcommands, options and args are removed from cmdline)
case "$cmd" in
    '')							# no cmd given
	mainproc
	err=$?
	;;
    -h)							# cmd -h
	helptext
	err=0
	;;
    -V)							# cmd -V
	echo "$scriptname (${package}) ${version}-${release}"
	err=0
	;;
    *)							# other cmds = error
	if [ "$verbose" != no ]; then
		echo "${ErrMsg[6]}" >&2
	else
		$errlog ${ErrMsg[6]}
	fi
	err=6
	;;
esac

exit $err
